package mylist;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertEquals;
import org.junit.Test;

// typical manual tests
public class MyListTest {
    @Test
    public void testLengthOnEmptyList() {
        final MyList<Object> emptyList = new EmptyList<Object>();
        assertEquals(0, emptyList.length());
    }

    public static <A> MyList<A> makeLength1List(final A contents) {
        final MyList<A> emptyList = new EmptyList<A>();
        return new NonEmptyList<A>(contents, emptyList);
    }

    public static <A> MyList<A> makeLength2List(final A first,
                                                final A second) {
        final MyList<A> length1 = makeLength1List(second);
        return new NonEmptyList<A>(first, length1);
    }
    
    @Test
    public void testLengthOnLength1List() {
        final MyList<Object> length1List = makeLength1List(new Object());
        assertEquals(1, length1List.length());
    }

    @Test
    public void testEqualsEmptyLists() {
        final MyList<Object> emptyList1 = new EmptyList<Object>();
        final MyList<Object> emptyList2 = new EmptyList<Object>();
        assertTrue(emptyList1.equals(emptyList2));
        assertTrue(emptyList2.equals(emptyList1));
    }

    @Test
    public void testEqualsLength1Lists() {
        final Object contents = new Object();
        final MyList<Object> list1 = makeLength1List(contents);
        final MyList<Object> list2 = makeLength1List(contents);
        
        assertTrue(list1.equals(list2));
        assertTrue(list2.equals(list1));
    }

    @Test
    public void testEqualsEmptyLength1Lists() {
        final MyList<Object> emptyList = new EmptyList<Object>();
        final MyList<Object> nonEmptyList = makeLength1List(new Object());

        assertFalse(emptyList.equals(nonEmptyList));
        assertFalse(nonEmptyList.equals(emptyList));
    }

    @Test
    public void testReverseEmptyList() {
        final MyList<Object> emptyList = new EmptyList<Object>();
        assertEquals(emptyList, emptyList.reverse());
    }

    @Test
    public void testReverseSingletonList() {
        final MyList<Object> list = makeLength1List(new Object());
        assertEquals(list, list.reverse());
    }

    @Test
    public void testReverseLength2List() {
        final Object contents1 = new Object();
        final Object contents2 = new Object();
        final MyList<Object> original = makeLength2List(contents1, contents2);
        final MyList<Object> expected = makeLength2List(contents2, contents1);
        assertEquals(expected, original.reverse());
    }
    
    @Test
    public void testAppendTwoEmptyLists() {
        final MyList<Object> emptyList = new EmptyList<Object>();
        assertEquals(emptyList, emptyList.append(emptyList));
    }

    @Test
    public void testAppendEmptyNonEmptyLists() {
        final Object contents = new Object();
        final MyList<Object> emptyList = new EmptyList<Object>();
        final MyList<Object> nonEmptyList = makeLength1List(contents);
        assertEquals(nonEmptyList, emptyList.append(nonEmptyList));
        assertEquals(nonEmptyList, nonEmptyList.append(emptyList));
    }

    @Test
    public void testAppendNonEmptyLists() {
        final Object contents1 = new Object();
        final Object contents2 = new Object();
        final MyList<Object> nonEmpty1 = makeLength1List(contents1);
        final MyList<Object> nonEmpty2 = makeLength1List(contents2);

        final MyList<Object> expected = makeLength2List(contents1,
                                                        contents2);
        assertEquals(expected, nonEmpty1.append(nonEmpty2));
    }
}

                                     
